/*

-Procedure specul ( locate the specular point on an ellipsoid )

-Abstract 

   This subroutine locates where on an ellipsoidal body
   a specular point would be seen.

-Copyright 

   Copyright (2001) California Institute of Technology.
   U.S. Government sponsorship acknowledged.

-Required_Reading

   None.

*/ 

   #include "SpiceUsr.h"
   #include "SpiceZmc.h"

   
   void specul ( SpiceDouble    arad,
                 SpiceDouble    brad,
                 SpiceDouble    crad,
                 SpiceDouble    sunpos    [3],
                 SpiceDouble    obspos    [3],
                 SpiceDouble    specpoint [3],
                 SpiceDouble   *spang,
                 SpiceBoolean  *found,
                 SpiceInt      *ier      ) 

   {

/*

-Brief I/O

   VARIABLE  I/O  DESCRIPTION
   --------  ---  --------------------------------------------------
   arad       I   Radius of ellipsoid along body-fixed frame x-axis. 
   brad       I   Radius of ellipsoid along body-fixed frame y-axis. 
   crad       I   Radius of ellipsoid along body-fixed frame z-axis. 
   sunpos     I   Vector from ellipsoid center to the sun.
   obspos     I   Vector from ellipsoid center to observer.
   specpoint  O   Vector from ellipsoid center to specular point.
   spang      O   Angle between surface normal and direction to
                  observer at the specular point.
   found      O   SPICETRUE if specular point is returned, SPICEFALSE
                  if there was an error condition or no convergence.
   ier        O   If found = SPICETRUE, this variable contains the   
                  number of iterations required for convergence.  If 
                  found = SPICEFALSE, the value returns an error code,
                  a negative integer, or if convergence was not obtained,
                  the number of iterations attempted.
 
                  error code        description 
                  ----------        -----------
                      >0            Number of iterations.

                      -1            Ray from observer to center of sun
                                    hits the body; no specular point. 

                      -2            Ray from center of body does not
                                    surface of body.

-&
*/

  
/*
   Local variables
*/

      #define            ESCALE   0.3
      #define            NITLIM   1000 
      #define            TOL      1.e-8

      SpiceBoolean       fnd;

      SpiceDouble        alt;
      SpiceDouble        center   [3];
      SpiceDouble        minrad; 
      SpiceDouble        normal   [3];
      SpiceDouble        pobsun   [3];
      SpiceDouble        rayint   [3];
      SpiceDouble        uobs     [3];
      SpiceDouble        up       [3];
      SpiceDouble        upos     [3];
      SpiceDouble        usun     [3];
      SpiceDouble        vmarch   [3];
      SpiceDouble        vscale;
      SpiceDouble        vtest; 

      SpiceInt           niter; 

/*
      Initialize variables.
*/

      *found = SPICEFALSE;
      vpack_c ( 0.0, 0.0, 0.0, center );


/*
      Pick the smallest ellipsoid radius and
      use it to scale the iterating vectors.
*/

      minrad = MinVal( MinVal(arad,brad), crad );
      vscale = ESCALE * minrad;

/*
      Construct the unit vector from the observer to the sun.
*/

      vsub_c ( sunpos, obspos, pobsun );  
      vhat_c ( pobsun, pobsun         );

/*
      Is the observer in shadow?  If so,
      a specular point is not seen.
*/

      surfpt_c ( obspos, pobsun, arad, brad, crad, rayint, &fnd );

/*
      If ray to sun hits ellipsoid, return to calling routine.
      The specular point is not visible in this case.
*/

      if ( fnd == SPICETRUE ) 
      {
         *found = SPICEFALSE;
         *ier = -1; 
         return;
      }

/*
      Obtain a surface point from which to start
      an interative search for the specular point.
*/

      vhat_c ( sunpos, usun );
      vhat_c ( obspos, uobs );  
      vadd_c ( usun, uobs, up );

      surfpt_c ( center, up, arad, brad, crad, specpoint, &fnd ); 


/*
      If the vector from the center of the body does not 
      intersect the surface, something is seriously wrong.
*/

      if ( fnd == SPICEFALSE )  
      {
         *found = SPICEFALSE;
         *ier = -2; 
         return;
      }  

      niter = 0; 

      while ( niter < NITLIM )  
      {
 

         niter++;

/*
         Calculate surface normal point at intercept point.
*/ 

         surfnm_c ( arad, brad, crad,  specpoint, normal );


/*
         Update pointing vector to observer
*/

         vsub_c ( obspos, specpoint, uobs );          
         vhat_c ( uobs,   uobs          );

/*
         Update pointing vector to sun.
*/

         vsub_c ( sunpos, specpoint, usun );
         vhat_c ( usun,   usun            );

/*
         Scale length of vectors to sun and to observer
         by the previously chosen scale factor.


         vscl_c ( vscale, uobs, sclobs );
         vscl_c ( vscale, usun, sclsun ); 
*/

/*
         Add observer and solar vectors, then
         scale the norm of the resultant.
         Call it the "march vector."
*/


         vadd_c ( uobs, usun, vmarch );
         vhat_c ( vmarch, vmarch         );

/*
         Compare the surface normal at the trial
         specular point with the sum of the 
         vectors to sun and observer.
*/

         vtest = vdist_c ( vmarch, normal ); 

/*
         Test for convergence
*/

         
         if ( vtest >= TOL ) 
         {

/*
            Scale the march vector.
*/

            vscl_c ( vscale, vmarch, vmarch );

/*
            Extend the march vector from the trial
            specular point. 
*/

            vadd_c ( specpoint, vmarch, vmarch );

/*
            Find new trial specular point.
*/

            nearpt_c ( vmarch, arad, brad, crad, specpoint, &alt );

         } else { 

            
/*       
            Solution has converged on a specular point.
*/

            *found = SPICETRUE;
            *ier = niter;

            *spang = vsep_c ( uobs, normal );

            return;
  
         }

      }
  
      *ier = niter;
      return;
}
